@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.95 2002/03/13 07:21:50 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.103 2002/08/09 11:43:41 lav Exp $ */
 
 #include <config.h>
 
@@ -31,6 +31,8 @@
 # include <netinet/tcp.h>
 #endif
 #include <errno.h>
+#include <ctype.h>
+#include <assert.h>
 #include "ascii_ctype.h"
 #include <fcntl.h>
 #include "LsCache.h"
@@ -41,6 +43,7 @@
 #include "DummyProto.h"
 #include "netrc.h"
 #include "ArgV.h"
+#include "ConnectionSlot.h"
 #ifdef WITH_MODULES
 # include "module.h"
 #endif
@@ -77,6 +80,7 @@ void FileAccess::Init()
    ascii=false;
    norest_manual=false;
    location=0;
+   suggested_filename=0;
 
    url=0;
 
@@ -111,6 +115,7 @@ FileAccess::FileAccess(const FileAccess *fa)
    hostname=xstrdup(fa->hostname);
    xfree(portname);
    portname=xstrdup(fa->portname);
+   vproto=xstrdup(fa->vproto);
 }
 
 FileAccess::~FileAccess()
@@ -234,8 +239,7 @@ int   FileAccess::CheckHangup(struct pollfd *pfd,int num)
 
 void FileAccess::NonBlock(int fd)
 {
-   int fl=0;
-   fcntl(fd,F_GETFL,&fl);
+   int fl=fcntl(fd,F_GETFL);
    fcntl(fd,F_SETFL,fl|O_NONBLOCK);
 }
 void FileAccess::CloseOnExec(int fd)
@@ -396,27 +400,10 @@ bool FileAccess::SameSiteAs(FileAccess *fa)
 
 const char *FileAccess::GetFileURL(const char *f,int flags)
 {
-   int len=1;
    const char *proto=GetVisualProto();
    if(proto[0]==0)
       return "";
 
-   len+=strlen(proto)+strlen("://");
-   if(user)
-   {
-      len+=strlen(user)*3+1;
-      if(pass)
-	 len+=strlen(pass)*3+1;
-   }
-   if(hostname)
-      len+=strlen(hostname)*3;
-   if(portname)
-      len+=1+strlen(portname)*3;
-   if(!f || (f[0]!='/' && f[0]!='~'))
-      f=dir_file(cwd?cwd:"~",f);
-   len+=1+strlen(f)*3;
-   url=(char*)xrealloc(url,len);
-
    ParsedURL u("");
 
    u.proto=(char*)proto;
@@ -427,9 +414,14 @@ const char *FileAccess::GetFileURL(const char *f,int flags)
    u.host=hostname;
    u.port=portname;
    if(!(flags&NO_PATH))
+   {
+      if(!f || (f[0]!='/' && f[0]!='~'))
+	 f=dir_file(cwd?cwd:"~",f);
       u.path=(char*)f;
+   }
 
-   u.Combine(url,home);
+   xfree(url);
+   url=u.Combine(home);
 
    return url;
 }
@@ -586,6 +578,9 @@ int last_element_is_doubledot(char *path,char *end)
 
 int FileAccess::device_prefix_len(const char *path)
 {
+   ResValue dp=Query("device-prefix",hostname);
+   if(dp.is_nil() || !dp.to_bool())
+      return 0;
    int i=0;
    while(path[i] && (is_ascii_alnum(path[i]) || strchr("$_-",path[i])))
       i++;
@@ -749,6 +744,30 @@ void FileAccess::Fatal(const char *e)
    SetError(FATAL,e);
 }
 
+void FileAccess::SetSuggestedFileName(const char *fn)
+{
+   xfree(suggested_filename);
+   suggested_filename=0;
+   if(fn==0)
+      return;
+
+   // don't allow subdirectories.
+   char *slash=strchr(fn,'/');
+   if(slash)
+      return;
+   slash=strchr(fn,'\\');
+   if(slash)
+      return;
+   for(int i=0; fn[i]; i++)
+   {
+      // don't allow control chars.
+      if(iscntrl((unsigned char)fn[i]))
+	 return;
+   }
+   if(!*fn)
+      return;
+   suggested_filename=xstrdup(fn);
+}
 
 FileAccess *SessionPool::pool[pool_size];
 
@@ -766,6 +785,7 @@ void SessionPool::Reuse(FileAccess *f)
    int i;
    for(i=0; i<pool_size; i++)
    {
+      assert(pool[i]!=f);
       if(pool[i]==0)
       {
 	 pool[i]=f;
@@ -876,6 +896,10 @@ void FileAccess::BumpEventTime(time_t t)
    if(event_time<t)
       event_time=t;
 }
+void FileAccess::SetTryTime(time_t t)
+{
+   try_time=t;
+}
 
 ResValue FileAccess::Query(const char *name,const char *closure)
 {
@@ -934,6 +958,12 @@ FileAccess *FileAccess::NextSameSite(FA *scan)
 
 FileAccess *FileAccess::New(const char *proto,const char *host)
 {
+   if(!strcmp(proto,"slot"))
+   {
+      FA *session=ConnectionSlot::FindSession(host);
+      return session?session->Clone():0;
+   }
+
    FA *session=Protocol::NewSession(proto);
    if(!session)
       return 0;
@@ -961,7 +991,8 @@ FileAccess *FileAccess::New(const ParsedURL *u,bool dummy)
 	 return 0;
       return new DummyNoProto(u->proto);
    }
-   s->Connect(u->host,u->port);
+   if(strcmp(u->proto,"slot"))
+      s->Connect(u->host,u->port);
    if(u->user)
       s->Login(u->user,u->pass);
    // path?
