@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.83 2002/01/10 09:52:25 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.89 2002/08/28 12:32:17 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -46,6 +46,7 @@
 #include "misc.h"
 #include "LsCache.h"
 #include "plural.h"
+#include "OutputJob.h"
 
 #define skip_threshold 0x1000
 #define debug(a) Log::global->Format a
@@ -144,6 +145,9 @@ int FileCopy::Do()
       }
       if(put->Broken())
       {
+	 get->Suspend();
+	 if(!put->Done())
+	    return m;
 	 debug((9,_("copy: put is broken\n")));
 	 if(fail_if_broken)
 	 {
@@ -545,7 +549,8 @@ long FileCopy::GetETA(off_t b)
 }
 const char *FileCopy::GetStatus()
 {
-   static char buf[256];
+   static char *buf;
+   xfree(buf); buf=0;
    const char *get_st=0;
    if(get)
       get_st=get->GetStatus();
@@ -553,11 +558,11 @@ const char *FileCopy::GetStatus()
    if(put)
       put_st=put->GetStatus();
    if(get_st && get_st[0] && put_st && put_st[0])
-      sprintf(buf,"[%s->%s]",get_st,put_st);
+      buf=xasprintf("[%s->%s]",get_st,put_st);
    else if(get_st && get_st[0])
-      sprintf(buf,"[%s]",get_st);
+      buf=xasprintf("[%s]",get_st);
    else if(put_st && put_st[0])
-      sprintf(buf,"[%s]",put_st);
+      buf=xasprintf("[%s]",put_st);
    else
       return "";
    return buf;
@@ -622,7 +627,7 @@ void FileCopyPeer::SetDate(time_t d)
 
 bool FileCopyPeer::Done()
 {
-   if(broken || Error())
+   if(Error())
       return true;
    if(eof && in_buffer==0)
    {
@@ -632,6 +637,8 @@ bool FileCopyPeer::Done()
 	 return done;
       return true;
    }
+   if(broken)
+      return true;
    return false;
 }
 
@@ -754,6 +761,7 @@ int FileCopyPeerFA::Do()
 	       {
 		  try_time=session->GetTryTime();
 		  retries=session->GetRetries();
+		  Log::global->Format(10,"try_time=%ld, retries=%d\n",try_time,retries);
 		  session->Close();
 		  if(can_seek && seek_pos>0)
 		     Seek(FILE_END);
@@ -870,8 +878,10 @@ void FileCopyPeerFA::OpenSession()
    session->SetFileURL(orig_url);
    if(mode==PUT)
    {
-      session->SetTryTime(try_time);
-      session->SetRetries(retries);
+      if(try_time!=0)
+	 session->SetTryTime(try_time);
+      if(retries!=0)
+	 session->SetRetries(retries);
       if(e_size!=NO_SIZE && e_size!=NO_SIZE_YET)
 	 session->SetSize(e_size);
       if(date!=NO_DATE && date!=NO_DATE_YET)
@@ -936,6 +946,8 @@ int FileCopyPeerFA::Get_LL(int len)
 	       SetError(_("Too many redirections"));
 	       return -1;
 	    }
+	    if(FAmode==FA::QUOTE_CMD)
+	       FAmode=FA::RETRIEVE;
 
 	    char *loc=alloca_strdup(loc_c);
 	    session->Close(); // loc_c is no longer valid.
@@ -1033,6 +1045,7 @@ int FileCopyPeerFA::Put_LL(const char *buf,int len)
       {
 	 try_time=session->GetTryTime();
 	 retries=session->GetRetries();
+	 Log::global->Format(10,"try_time=%ld, retries=%d\n",try_time,retries);
 	 session->Close();
 	 if(can_seek && seek_pos>0)
 	    Seek(FILE_END);
@@ -1497,6 +1510,8 @@ int FileCopyPeerFDStream::Put_LL(const char *buf,int len)
       if(errno==EPIPE)
       {
 	 broken=true;
+	 in_buffer=0;
+	 eof=true;
 	 return -1;
       }
       if(stream->NonFatalError(errno))
@@ -1550,31 +1565,6 @@ FileCopyPeerFDStream *FileCopyPeerFDStream::NewGet(const char *file)
 }
 
 
-// FileCopyPeerString
-#undef super
-#define super FileCopyPeer
-FileCopyPeerString::FileCopyPeerString(const char *s, int len)
-   : super(GET)
-{
-   if(len==-1)
-      len=strlen(s);
-   Put(s,len);
-   eof=true;
-   pos=0;
-   can_seek=true;
-   can_seek0=true;
-}
-FileCopyPeerString::~FileCopyPeerString()
-{
-}
-void FileCopyPeerString::Seek(off_t new_pos)
-{
-   assert(new_pos!=FILE_END);
-   UnSkip(pos-new_pos);
-   super::Seek(new_pos);
-   pos=new_pos;
-}
-
 // FileCopyPeerDirList
 FileCopyPeerDirList::FileCopyPeerDirList(FA *s,ArgV *v)
    : FileCopyPeer(GET)
@@ -1620,5 +1610,77 @@ int FileCopyPeerDirList::Do()
    return MOVED;
 }
 
+FileCopyPeerOutputJob::FileCopyPeerOutputJob(OutputJob *new_o)
+   : FileCopyPeer(PUT)
+{
+   o=new_o;
+   DontCopyDate();
+}
+
+int FileCopyPeerOutputJob::Put_LL(const char *buf,int len)
+{
+   off_t io_at=pos;
+   if(GetRealPos()!=io_at) // GetRealPos can alter pos.
+      return 0;
+
+   if(len==0 && eof)
+      return 0;
+
+   if(o->Full())
+      return 0;
+
+   o->Put(buf,len);
+
+   seek_pos+=len; // mainly to indicate that there was some output.
+   return len;
+}
+
+int FileCopyPeerOutputJob::Do()
+{
+   if(o->Error())
+   {
+      broken=true;
+      return MOVED;
+   }
+
+   if(eof && !in_buffer)
+   {
+      done=true;
+      return MOVED;
+   }
+
+   int m=STALL;
+
+   if(!write_allowed)
+      return m;
+
+   while(in_buffer>0)
+   {
+      int res=Put_LL(buffer+buffer_ptr,in_buffer);
+      if(res>0)
+      {
+	 in_buffer-=res;
+	 buffer_ptr+=res;
+	 m=MOVED;
+      }
+      if(res<0)
+	 return MOVED;
+      if(res==0)
+	 break;
+   }
+   return m;
+}
+
+void FileCopyPeerOutputJob::Fg()
+{
+   o->Fg();
+   FileCopyPeer::Fg();
+}
+void FileCopyPeerOutputJob::Bg()
+{
+   o->Bg();
+   FileCopyPeer::Bg();
+}
+
 // special pointer to creator of ftp/ftp copier. It is init'ed in Ftp class.
 FileCopy *(*FileCopy::fxp_create)(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
