@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.h,v 1.42 2001/11/06 13:15:32 lav Exp $ */
+/* $Id: FileCopy.h,v 1.50 2002/07/10 08:48:51 lav Exp $ */
 
 /*
   classes defined here:
@@ -26,8 +26,8 @@
    FileCopyPeer
    +FileCopyPeerFA
    +FileCopyPeerFDStream
-   +FileCopyPeerString
    \FileCopyPeerList
+   +FileCopyPeerOutputJob
 */
 
 #ifndef FILECOPY_H
@@ -55,6 +55,9 @@ protected:
    bool date_set;
    bool do_set_date;
    bool removing;
+   bool file_removed;
+
+   bool done;
 
    bool ascii;
    bool use_cache;
@@ -109,7 +112,7 @@ public:
    virtual pid_t GetProcGroup() { return 0; }
    virtual void Kill(int sig) {}
 
-   virtual void RemoveFile() {}
+   virtual void RemoveFile() { file_removed=true; }
    virtual void NeedSeek() {} // fd is shared, seek before access.
 
    void CannotSeek(int p)
@@ -122,9 +125,11 @@ public:
    virtual void Fg() {}
    virtual void Bg() {}
 
-   void AllowWrite() { write_allowed=true; }
+   void AllowWrite(bool y=true) { write_allowed=y; }
    bool WriteAllowed() { return write_allowed; }
    bool WritePending() { return mode==PUT && Size()>0; }
+
+   bool FileRemoved() { return file_removed; }
 };
 
 class FileCopy : public SMTask
@@ -161,7 +166,9 @@ private:
    Time end_time;
 
    bool fail_if_cannot_seek;
+   bool fail_if_broken;
    bool remove_source_later;
+   bool remove_target_first;
 
    Buffer *line_buffer;
    int  line_buffer_max;
@@ -211,10 +218,13 @@ public:
 
    void DontCopyDate() { put->DontCopyDate(); }
    void Ascii() { get->Ascii(); put->Ascii(); }
+   void DontFailIfBroken() { fail_if_broken=false; }
    void FailIfCannotSeek() { fail_if_cannot_seek=true; }
    void SetRange(off_t s,off_t lim) { get->SetRange(s,lim); put->SetRange(s,lim); }
    void RemoveSourceLater() { remove_source_later=true; }
+   void RemoveTargetFirst() { remove_target_first=true; put->Resume(); put->RemoveFile(); }
    void LineBuffered(int size=0x1000);
+   bool IsLineBuffered() const { return line_buffer; }
 
    FileCopy(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
    void Init();
@@ -230,7 +240,7 @@ public:
    static FileCopy *New(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
    static FileCopy *(*fxp_create)(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
 
-   void AllowWrite() { put->AllowWrite(); }
+   void AllowWrite(bool y=true) { if(put) put->AllowWrite(y); }
    bool WriteAllowed() { return !put || put->WriteAllowed(); }
    bool WritePending() { return put && put->WritePending(); }
 };
@@ -275,6 +285,7 @@ public:
    void DontReuseSession() { reuse_later=false; }
 
    const char *GetStatus();
+   const char *GetProto() { return session->GetProto(); }
 
    bool NeedSizeDateBeforehand() { return session->NeedSizeDateBeforehand(); }
    void RemoveFile();
@@ -309,12 +320,11 @@ protected:
 public:
    FileCopyPeerFDStream(FDStream *o,direction m);
    int Do();
-   bool Done();
    bool IOReady();
    void Seek(off_t new_pos);
    FgData *GetFgData(bool fg);
    pid_t GetProcGroup() { return stream->GetProcGroup(); }
-   void Kill(int sig) { stream->Kill(sig); }
+   void Kill(int sig);
 
    void DontDeleteStream() { delete_stream=false; }
    void DontCreateFgData() { create_fg_data=false; }
@@ -322,20 +332,12 @@ public:
    void RemoveFile();
    void SetBase(off_t b) { seek_base=b; }
 
+   const char *GetStatus();
+
    static FileCopyPeerFDStream *NewPut(const char *file,bool cont=false);
    static FileCopyPeerFDStream *NewGet(const char *file);
 };
 
-class FileCopyPeerString : public FileCopyPeer
-{
-protected:
-   ~FileCopyPeerString();
-
-public:
-   FileCopyPeerString(const char *s,int len=-1);
-   void Seek(off_t new_pos);
-};
-
 class FileCopyPeerDirList : public FileCopyPeer
 {
 private:
@@ -353,6 +355,22 @@ public:
    void Fg() { session->SetPriority(1); }
    void Bg() { session->SetPriority(0); }
    const char *GetStatus() { return session->CurrentStatus(); }
+   void UseColor(bool c=true) { if(dl) dl->UseColor(c); }
+};
+
+class OutputJob;
+
+class FileCopyPeerOutputJob : public FileCopyPeer
+{
+   OutputJob *o;
+   int Put_LL(const char *buf,int len);
+
+public:
+   FileCopyPeerOutputJob(OutputJob *o);
+
+   int Do();
+   void Fg();
+   void Bg();
 };
 
 #endif
