@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.32 2001/12/14 19:06:00 lav Exp $ */
+/* $Id: FileSet.cc,v 1.33 2002/07/31 14:35:50 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -290,6 +290,18 @@ void FileSet::SubtractOlderThan(time_t t)
       }
    }
 }
+void FileSet::SubtractDirs()
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(files[i]->defined&FileInfo::TYPE
+      && files[i]->filetype==FileInfo::DIRECTORY)
+      {
+	 Sub(i);
+	 i--;
+      }
+   }
+}
 
 void FileSet::ExcludeDots()
 {
