@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FindJob.cc,v 1.18 2001/11/09 13:36:27 lav Exp $ */
+/* $Id: FindJob.cc,v 1.29 2002/07/31 11:28:00 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -27,58 +27,69 @@
 #include "FindJob.h"
 #include "CmdExec.h"
 #include "misc.h"
+#include "GetFileInfo.h"
+#include "url.h"
+#include "PatternSet.h"
 
 #define top (*stack[stack_ptr])
+#define super SessionJob
 
 int FinderJob::Do()
 {
    int m=STALL;
-   int res;
    prf_res pres;
    Job *j;
 
    switch(state)
    {
-   case INIT:
-      // nothing to do - just fall to CD
+   case START_INFO:
+   {
+      if(stack_ptr==-1)
+      {
+	 ParsedURL u(dir,true);
+	 if(u.proto)
+	 {
+	    session=FileAccess::New(&u);
+	    session->SetPriority(fg?1:0);
+	    init_dir=xstrdup(session->GetCwd());
+	    Down(u.path?u.path:init_dir);
+	 }
+      }
 
+      /* If we're not validating, and this is an argument (first-level path),
+       * pretend the file exists. */
+      if((file_info_need|FileInfo::NAME) == FileInfo::NAME &&
+	    !validate_args && stack_ptr == -1)
       {
-	 const char *cd_dir=dir;
-	 if(stack_ptr>=0)
-	    cd_dir=dir_file(dir_file(start_dir,top.path),dir);
-	 session->Chdir(cd_dir,false);
+	 FileSet *fs = new FileSet();
+	 fs->Add(new FileInfo(dir));
+	 Push(fs);
+	 state=LOOP;
+	 return MOVED;
       }
-      state=CD;
-      m=MOVED;
-   case CD:
-      res=session->Done();
-      if(res==session->IN_PROGRESS)
-	 return m;
-      if(res==session->OK)
-      {
-	 session->Close();
-	 goto pre_INFO;
-      }
-      // cd error
-      if(!quiet)
-	 eprintf("%s: cd %s: %s\n",op,dir,session->StrError(res));
-      errors++;
-      depth_done=true;
-      state=LOOP;
-      return MOVED;
 
-   pre_INFO:
-      li=session->MakeListInfo();
-      if(li==0)
-      {
-	 //FIXME
-	 abort();
-      }
-      li->Need(file_info_need|FileInfo::NAME|FileInfo::TYPE);
+      /* The first time we get here (stack_ptr == -1), dir is an actual
+       * argument, so it might be a file.  (Every other time, it's guaranteed
+       * to be a directory.)  Set show_dirs to true, so it'll end up actually
+       * being on the stack, with type information. */
+      li=new GetFileInfo(session, dir, stack_ptr == -1);
+
+      /* Prepend for the argument level entry only: */
+      if(stack_ptr != -1)
+	 li->DontPrependPath();
+
+      int need = file_info_need|FileInfo::NAME;
+
+      /* We only explicitely need the type if we're recursing further. */
+      if(stack_ptr+1 < maxdepth)
+	 need |= FileInfo::TYPE;
+
+      li->Need(need);
       if(use_cache)
 	 li->UseCache();
       state=INFO;
       m=MOVED;
+   }
    case INFO:
       if(!li->Done())
 	 return m;
@@ -93,11 +104,15 @@ int FinderJob::Do()
 	 state=LOOP;
 	 return MOVED;
       }
+
+      if(stack_ptr != -1 && li->WasDirectory())
+	 Enter(dir);
+
       Push(li->GetResult());
+      top.fset->rewind();
+
       Delete(li);
       li=0;
-
-      top.fset->rewind();
       state=LOOP;
       m=MOVED;
    case LOOP:
@@ -106,6 +121,9 @@ int FinderJob::Do()
 	 Up();
 	 return MOVED;
       }
+
+      session->SetCwd(init_dir);
+      session->Chdir(top.path,false);
       // at this point either is true:
       // 1. we just process another file (!depth_done)
       // 2. we just returned from a subdir (depth_done)
@@ -115,7 +133,6 @@ int FinderJob::Do()
 	 if((f->defined&f->TYPE) && f->filetype==f->DIRECTORY)
 	 {
 	    Down(f->name);
-	    Enter(f->name);
 	    return MOVED;
 	 }
       }
@@ -154,7 +171,6 @@ int FinderJob::Do()
 	 {
 	    top.fset->next();
 	    Down(f->name);
-	    Enter(f->name);
 	    return MOVED;
 	 }
       }
@@ -186,7 +202,10 @@ void FinderJob::Up()
       Finish();
       return;
    }
-   Exit();
+   /* stack[0] is the dir entry for the argument (ie. ls -d dir), and
+    * stack[1] is the contents (ls dir); don't exit for the first. */
+   if(stack_ptr)
+      Exit();
    delete stack[stack_ptr--];
    if(stack_ptr==-1)
       goto done;
@@ -207,15 +226,17 @@ void FinderJob::Push(FileSet *fset)
       stack=(place**)xrealloc(stack,sizeof(*stack)*stack_allocated);
    }
 
+   if(stack_ptr != 0)
+      fset->ExcludeDots(); /* don't need . and .. (except for stack[0]) */
+
    const char *new_path="";
    if(old_path) // the first path will be empty
-      new_path=dir_file(old_path,dir);
+      new_path=alloca_strdup(dir_file(old_path,dir));
 
-   fset->ExcludeDots(); // don't need . and ..
    /* matching exclusions don't include the path, so they operate
     * on the filename portion only */
    if(exclude)
-      fset->Exclude(0, exclude,0);
+      fset->Exclude(0, exclude);
    stack[stack_ptr]=new place(new_path,fset);
 
    /* give a chance to operate on the list as a whole, and
@@ -236,8 +257,12 @@ FinderJob::place::~place()
 
 void FinderJob::Down(const char *p)
 {
-   dir=p;
-   state=INIT;
+#ifdef FIND_DEBUG
+printf("Down(%s)\n",p);
+#endif
+   xfree(dir);
+   dir=xstrdup(p);
+   state=START_INFO;
 }
 
 FinderJob::prf_res FinderJob::ProcessFile(const char *d,const FileInfo *f)
@@ -247,8 +272,10 @@ FinderJob::prf_res FinderJob::ProcessFile(const char *d,const FileInfo *f)
 
 void FinderJob::Init()
 {
+   orig_session=0;
+   orig_init_dir=0;
+
    op="find";
-   start_dir=0;
    init_dir=0;
    dir=0;
    errors=0;
@@ -265,12 +292,13 @@ void FinderJob::Init()
 
    file_info_need=0;
    use_cache=true;
+   validate_args=false;
 
    quiet=false;
    maxdepth=-1;
    exclude=0;
 
-   state=INIT;
+   state=START_INFO;
 }
 
 FinderJob::FinderJob(FileAccess *s)
@@ -278,25 +306,36 @@ FinderJob::FinderJob(FileAccess *s)
 {
    Init();
    init_dir=xstrdup(session->GetCwd());
+   orig_session=session;
+   orig_init_dir=init_dir;
 }
 
 void FinderJob::NextDir(const char *d)
 {
-   session->Chdir(init_dir,false); // no verification
-   xfree(start_dir);
-   start_dir=xstrdup(dir_file(session->GetCwd(),d));
-   Down(start_dir);
-   Enter(d); /* tell derived that we entered the base path */
+   if(session!=orig_session)
+   {
+      Reuse(session);
+      session=orig_session;
+      assert(orig_init_dir!=init_dir);
+      xfree(init_dir);
+      init_dir=orig_init_dir;
+   }
+   session->SetCwd(init_dir);
+   Down(d);
 }
 
 FinderJob::~FinderJob()
 {
    while(stack_ptr>=0)
       Up();
+   if(orig_session!=session)
+      Reuse(orig_session);
+   if(orig_init_dir!=init_dir)
+      xfree(orig_init_dir);
    xfree(stack);
-   xfree(start_dir);
    xfree(init_dir);
-   xfree(exclude);
+   delete exclude;
+   xfree(dir);
    Delete(li);
 }
 
@@ -322,9 +361,9 @@ void FinderJob::ShowRunStatus(StatusLine *sl)
    }
 }
 
-void FinderJob::PrintStatus(int v)
+void FinderJob::PrintStatus(int v,const char *prefix)
 {
-   SessionJob::PrintStatus(v);
+   SessionJob::PrintStatus(v,prefix);
 
    char *path=0;
    switch(state)
@@ -332,8 +371,7 @@ void FinderJob::PrintStatus(int v)
    case INFO:
       if(stack_ptr>=0)
 	 path=top.path;
-      printf("\t%s: %s [%s]\n",dir_file(path,dir),li->Status(),
-			   session->CurrentStatus());
+      printf("\t%s: %s\n",dir_file(path,dir),li->Status());
       break;
    case WAIT:
       break;
@@ -342,6 +380,19 @@ void FinderJob::PrintStatus(int v)
    }
 }
 
+void FinderJob::Fg()
+{
+   super::Fg();
+   if(orig_session!=session)
+      orig_session->SetPriority(1);
+}
+void FinderJob::Bg()
+{
+   if(orig_session!=session)
+      orig_session->SetPriority(0);
+   super::Bg();
+}
+
 // FinderJob_List implementation
 // find files and write list to a stream
 FinderJob::prf_res FinderJob_List::ProcessFile(const char *d,const FileInfo *fi)
@@ -357,7 +408,16 @@ FinderJob::prf_res FinderJob_List::ProcessFile(const char *d,const FileInfo *fi)
       fg_data=buf->GetFgData(fg);
    if(buf->Size()>0x10000)
       return PRF_LATER;
-   buf->Put(dir_file(d,fi->name));
+   if(ProcessingURL())
+   {
+      char *old_cwd=alloca_strdup(session->GetCwd());
+      session->SetCwd(init_dir);
+      session->Chdir(dir_file(d,fi->name),false);
+      buf->Put(session->GetConnectURL());
+      session->SetCwd(old_cwd);
+   }
+   else
+      buf->Put(dir_file(d,fi->name));
    if((fi->defined&fi->TYPE) && fi->filetype==fi->DIRECTORY)
       buf->Put("/");
    buf->Put("\n");
@@ -370,11 +430,13 @@ FinderJob_List::FinderJob_List(FileAccess *s,ArgV *a,FDStream *o)
    show_sl = !o->usesfd(1);
    buf=new IOBufferFDStream(o,IOBuffer::PUT);
    NextDir(a->getcurr());
+   ValidateArgs();
 }
 
 FinderJob_List::~FinderJob_List()
 {
    Delete(buf);
+   delete args;
 }
 
 void FinderJob_List::Finish()
@@ -387,8 +449,10 @@ void FinderJob_List::Finish()
    NextDir(d);
 }
 
+#if 0 // unused
 // FinderJob_Cmd implementation
 // process directory tree
+#undef super
 #define super FinderJob
 
 FinderJob::prf_res FinderJob_Cmd::ProcessFile(const char *d,const FileInfo *f)
@@ -397,7 +461,6 @@ FinderJob::prf_res FinderJob_Cmd::ProcessFile(const char *d,const FileInfo *f)
 #define ISLINK ((f->defined&f->TYPE) && f->filetype==f->SYMLINK)
 
    FileAccess *s=session->Clone();
-   s->Chdir(dir_file(start_dir,d),false);
 
    CmdExec *exec=new CmdExec(s);
    exec->SetParentFg(this);
@@ -478,29 +541,6 @@ FinderJob_Cmd::~FinderJob_Cmd()
 
 void FinderJob_Cmd::Finish()
 {
-   if(cmd==RM)
-   {
-      if(removing_last)
-	 removing_last=false;
-      else
-      {
-	 /* remove the specified directory at the last */
-	 session->Chdir(init_dir,false); // to leave the directory
-	 CmdExec *exec=new CmdExec(session->Clone());
-	 exec->SetParentFg(this);
-	 exec->FeedCmd("rmdir ");
-	 if(quiet)
-	    exec->FeedCmd("-f ");
-	 exec->FeedCmd("-- ");
-	 exec->FeedQuoted(start_dir);
-	 exec->FeedCmd("\n");
-	 AddWaiting(exec);
-	 removing_last=true;
-	 state=WAIT; // it will wait for termination, try to process
-		     // next file and call Finish() again
-	 return;
-      }
-   }
    char *d=args->getnext();
    if(!d)
       return;
@@ -511,4 +551,4 @@ int FinderJob_Cmd::Done()
 {
    return FinderJob::Done() && args->getcurr()==0;
 }
-
+#endif
