@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -260,7 +260,7 @@ int Fish::Do()
       }
       goto usual_return;
    case FILE_RECV:
-      if(recv_buf->Size()>=rate_limit->BytesAllowed())
+      if(recv_buf->Size()>=rate_limit->BytesAllowedToGet())
       {
 	 recv_buf->Suspend();
 	 Timeout(1000);
@@ -943,7 +943,7 @@ int Fish::Read(void *buf,int size)
 	 }
       }
 
-      int bytes_allowed=rate_limit->BytesAllowed();
+      int bytes_allowed=rate_limit->BytesAllowedToGet();
       if(size1>bytes_allowed)
 	 size1=bytes_allowed;
       if(size1==0)
@@ -965,7 +965,7 @@ int Fish::Read(void *buf,int size)
       recv_buf->Skip(size);
       pos+=size;
       real_pos+=size;
-      rate_limit->BytesUsed(size);
+      rate_limit->BytesGot(size);
       retries=0;
       return size;
    }
@@ -986,7 +986,7 @@ int Fish::Write(const void *buf,int size)
       return DO_AGAIN;
 
    {
-      int allowed=rate_limit->BytesAllowed();
+      int allowed=rate_limit->BytesAllowedToPut();
       if(allowed==0)
 	 return DO_AGAIN;
       if(size+send_buf->Size()>allowed)
@@ -1000,7 +1000,7 @@ int Fish::Write(const void *buf,int size)
       return 0;
    send_buf->Put((char*)buf,size);
    retries=0;
-   rate_limit->BytesUsed(size);
+   rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
    return(size);
@@ -1139,7 +1139,7 @@ DirList *Fish::MakeDirList(ArgV *args)
 {
    return new FishDirList(args,this);
 }
-#include "Glob.h"
+#include "FileGlob.h"
 Glob *Fish::MakeGlob(const char *pattern)
 {
    return new GenericGlob(this,pattern);
@@ -1191,15 +1191,7 @@ int FishDirList::Do()
    if(b==0) // eof
    {
       buf->PutEOF();
-
-      const char *cache_buffer;
-      int cache_buffer_size;
-      ubuf->GetSaved(&cache_buffer,&cache_buffer_size);
-      if(cache_buffer && cache_buffer_size>0)
-      {
-	 LsCache::Add(session,pattern,FA::LONG_LIST,
-		      cache_buffer,cache_buffer_size);
-      }
+      LsCache::Add(session,pattern,FA::LONG_LIST, ubuf);
 
       return MOVED;
    }
