@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.144 2002/04/08 08:23:55 lav Exp $ */
+/* $Id: Http.cc,v 1.155 2002/09/05 09:37:51 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -344,9 +344,9 @@ void Http::SendMethod(const char *method,const char *efile)
 void Http::SendBasicAuth(const char *tag,const char *user,const char *pass)
 {
    /* Basic scheme */
-   char *buf=(char*)alloca(strlen(user)+1+strlen(pass)+1);
+   char *buf=string_alloca(strlen(user)+1+strlen(pass)+1);
    sprintf(buf,"%s:%s",user,pass);
-   char *buf64=(char*)alloca(base64_length(strlen(buf))+1);
+   char *buf64=string_alloca(base64_length(strlen(buf))+1);
    base64_encode(buf,buf64,strlen(buf));
    Send("%s: Basic %s\r\n",tag,buf64);
 }
@@ -391,33 +391,26 @@ void Http::SendRequest(const char *connection,const char *f)
    url::encode_string(cwd,ecwd,URL_PATH_UNSAFE);
    int efile_len;
 
-   char *pfile=(char*)alloca(4+3+xstrlen(user)*6+3+xstrlen(pass)*3+1+
-			      strlen(hostname)*3+1+strlen(cwd)*3+1+
-			      strlen(efile)+1+6+1);
+   char *pfile=string_alloca(4+3+xstrlen(user)*6+3+xstrlen(pass)*3+1+
+			      strlen(hostname)*3+1+xstrlen(portname)*3+1+
+			      strlen(cwd)*3+1+strlen(efile)+1+6+1);
 
    if(proxy && !https)
    {
       const char *proto="http";
-      if(https)
-	 proto="https";
       if(hftp)
-      {
-	 if(user && pass)
-	 {
-	    strcpy(pfile,"ftp://");
-	    url::encode_string(user,pfile+strlen(pfile),URL_USER_UNSAFE);
-	    if(!QueryBool("use-authorization",proxy))
-	    {
-	       strcat(pfile,":");
-	       url::encode_string(pass,pfile+strlen(pfile),URL_PASS_UNSAFE);
-	    }
-	    strcat(pfile,"@");
-	    url::encode_string(hostname,pfile+strlen(pfile),URL_HOST_UNSAFE);
-	    goto add_path;
-	 }
 	 proto="ftp";
-      }
       sprintf(pfile,"%s://",proto);
+      if(hftp && user && pass)
+      {
+	 url::encode_string(user,pfile+strlen(pfile),URL_USER_UNSAFE);
+	 if(!QueryBool("use-authorization",proxy))
+	 {
+	    strcat(pfile,":");
+	    url::encode_string(pass,pfile+strlen(pfile),URL_PASS_UNSAFE);
+	 }
+	 strcat(pfile,"@");
+      }
       url::encode_string(hostname,pfile+strlen(pfile),URL_HOST_UNSAFE);
       if(portname)
       {
@@ -430,8 +423,6 @@ void Http::SendRequest(const char *connection,const char *f)
       pfile[0]=0;
    }
 
-add_path:
-
    char *path_base=pfile+strlen(pfile);
 
    if(efile[0]=='/')
@@ -690,21 +681,56 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 seen_ranges_bytes=true;
       return;
    }
-   if(!strcasecmp(name,"Set-Cookie")
-   && !hftp && QueryBool("set-cookies",hostname))
-      SetCookie(value);
+   if(!strcasecmp(name,"Set-Cookie"))
+   {
+      if(!hftp && QueryBool("set-cookies",hostname))
+	 SetCookie(value);
+      return;
+   }
+   if(!strcasecmp(name,"Content-Disposition"))
+   {
+      const char *filename=strstr(value,"filename=");
+      if(!filename)
+	 return;
+      if(*filename=='"')
+      {
+	 filename++;
+	 char *filename1=string_alloca(strlen(filename));
+	 while(*filename && *filename!='"')
+	 {
+	    if(*filename=='\\' && filename[1])
+	       filename++;
+	    *filename1++=*filename++;
+	 }
+	 filename=filename1;
+      }
+      else
+      {
+	 int end=strcspn(filename,"()<>@,;:\\\"/[]?={} \t");
+	 char *filename1=alloca_strdup(filename);
+	 filename1[end]=0;
+	 filename=filename1;
+      }
+      SetSuggestedFileName(filename);
+      return;
+   }
 }
 
-static const char *find_eol(const char *str,int len)
+static
+const char *find_eol(const char *p,int len,int *eol_size)
 {
-   const char *p=str;
-   for(int i=0; i<len-1; i++,p++)
+   *eol_size=1;
+   for(int i=0; i<len; i++,p++)
    {
-      if(p[1]=='\n' && p[0]=='\r')
+      if(p[0]=='\n')
 	 return p;
-      if(p[1]!='\r')
-	 p++,i++; // fast skip
+      if(i+1<len && p[1]=='\n' && p[0]=='\r')
+      {
+	 *eol_size=2;
+	 return p;
+      }
    }
+   *eol_size=0;
    return 0;
 }
 
@@ -1004,10 +1030,19 @@ int Http::Do()
       {
 	 if((mode==STORE || post) && status_code && !H_20X(status_code))
 	    goto pre_RECEIVING_BODY;   // assume error.
+      handle_buf_error:
 	 if(send_buf->Error())
+	 {
 	    DebugPrint("**** ",send_buf->ErrorText(),0);
+	    if(send_buf->ErrorFatal())
+	       SetError(FATAL,send_buf->ErrorText());
+	 }
 	 if(recv_buf->Error())
+	 {
 	    DebugPrint("**** ",recv_buf->ErrorText(),0);
+	    if(recv_buf->ErrorFatal())
+	       SetError(FATAL,recv_buf->ErrorText());
+	 }
 	 Disconnect();
 	 return MOVED;
       }
@@ -1028,13 +1063,14 @@ int Http::Do()
       }
       if(len>0)
       {
-	 const char *eol=find_eol(buf,len);
+	 int eol_size;
+	 const char *eol=find_eol(buf,len,&eol_size);
 	 if(eol)
 	 {
 	    if(eol==buf)
 	    {
 	       DebugPrint("<--- ","",4);
-	       recv_buf->Skip(2);
+	       recv_buf->Skip(eol_size);
 	       if(tunnel_state==TUNNEL_WAITING)
 	       {
 		  if(H_20X(status_code))
@@ -1056,6 +1092,13 @@ int Http::Do()
 		  state=DONE;
 		  return MOVED;
 	       }
+	       if(H_CONTINUE(status_code))
+	       {
+		  xfree(status);
+		  status=0;
+		  status_code=0;
+		  return MOVED;
+	       }
 	       if(mode==ARRAY_INFO)
 	       {
 		  // we'll have to receive next header
@@ -1085,13 +1128,6 @@ int Http::Do()
 	       }
 	       else if(mode==STORE)
 	       {
-		  if(H_CONTINUE(status_code))
-		  {
-		     xfree(status);
-		     status=0;
-		     status_code=0;
-		     return MOVED;
-		  }
 		  if((sent_eot || pos==entity_size) && H_20X(status_code))
 		  {
 		     state=DONE;
@@ -1116,7 +1152,7 @@ int Http::Do()
 	    memcpy(line,buf,len);
 	    line[len]=0;
 
-	    recv_buf->Skip(len+2);
+	    recv_buf->Skip(len+eol_size);
 
 	    DebugPrint("<--- ",line,4);
 	    m=MOVED;
@@ -1213,11 +1249,45 @@ int Http::Do()
 
       if(!H_20X(status_code))
       {
-	 char *err=(char*)alloca(strlen(status)+strlen(file)+strlen(cwd)+xstrlen(location)+20);
+	 char *err=string_alloca(strlen(status)+strlen(file)+strlen(cwd)+xstrlen(location)+20);
 	 int code=NO_FILE;
 
 	 if(H_REDIRECTED(status_code))
 	 {
+	    if(location && !url::is_url(location)
+	    && mode==QUOTE_CMD && !strncasecmp(file,"POST ",5))
+	    {
+	       const char *the_file=file;
+
+	       const char *scan=file+5;
+	       while(*scan==' ')
+		  scan++;
+	       char *the_post_file=alloca_strdup(scan);
+	       char *space=strchr(the_post_file,' ');
+	       if(space)
+		  *space=0;
+	       the_file=the_post_file;
+
+	       char *new_location=alloca_strdup2(GetConnectURL(),
+				    strlen(the_file)+strlen(location));
+	       int p_ind=url::path_index(new_location);
+	       if(location[0]=='/')
+		  strcpy(new_location+p_ind,location);
+	       else
+	       {
+		  if(the_file[0]=='/')
+		     strcpy(new_location+p_ind,the_file);
+		  else
+		  {
+		     char *slash=strrchr(new_location,'/');
+		     strcpy(slash+1,the_file);
+		  }
+		  char *slash=strrchr(new_location,'/');
+		  strcpy(slash+1,location);
+	       }
+	       xfree(location);
+	       location=xstrdup(new_location);
+	    }
 	    sprintf(err,"%s (%s -> %s)",status+status_consumed,file,
 				    location?location:"nowhere");
 	    code=FILE_MOVED;
@@ -1259,15 +1329,8 @@ int Http::Do()
       m=MOVED;
    case RECEIVING_BODY:
       if(recv_buf->Error() || send_buf->Error())
-      {
-	 if(send_buf->Error())
-	    DebugPrint("**** ",send_buf->ErrorText(),0);
-	 if(recv_buf->Error())
-	    DebugPrint("**** ",recv_buf->ErrorText(),0);
-	 Disconnect();
-	 return MOVED;
-      }
-      if(recv_buf->Size()>=rate_limit->BytesAllowed())
+	 goto handle_buf_error;
+      if(recv_buf->Size()>=rate_limit->BytesAllowedToGet())
       {
 	 recv_buf->Suspend();
 	 Timeout(1000);
@@ -1367,6 +1430,8 @@ int Http::Read(void *buf,int size)
       if(recv_buf->Size()==0 && recv_buf->Error())
       {
 	 DebugPrint("**** ",recv_buf->ErrorText(),0);
+	 if(recv_buf->ErrorFatal())
+	    SetError(FATAL,recv_buf->ErrorText());
 	 Disconnect();
 	 return DO_AGAIN;
       }
@@ -1451,7 +1516,7 @@ int Http::Read(void *buf,int size)
 	    size1=body_size-bytes_received;
       }
 
-      int bytes_allowed=rate_limit->BytesAllowed();
+      int bytes_allowed=rate_limit->BytesAllowedToGet();
       if(size1>bytes_allowed)
 	 size1=bytes_allowed;
       if(size1==0)
@@ -1479,7 +1544,7 @@ int Http::Read(void *buf,int size)
       bytes_received+=size;
       if(chunked)
 	 chunk_pos+=size;
-      rate_limit->BytesUsed(size);
+      rate_limit->BytesGot(size);
       retries=0;
       return size;
    }
@@ -1513,7 +1578,7 @@ int Http::Write(const void *buf,int size)
       return DO_AGAIN;
 
    {
-      int allowed=rate_limit->BytesAllowed();
+      int allowed=rate_limit->BytesAllowedToPut();
       if(allowed==0)
 	 return DO_AGAIN;
       if(size>allowed)
@@ -1537,7 +1602,7 @@ int Http::Write(const void *buf,int size)
       return error_code;
    }
    retries=0;
-   rate_limit->BytesUsed(res);
+   rate_limit->BytesPut(res);
    pos+=res;
    real_pos+=res;
    return(res);
@@ -1626,6 +1691,9 @@ void Http::Reconfig(const char *name)
 	    p=ResMgr::Query("https:proxy",c);
 	 else
 	    p=Query("proxy",c);
+	 // if no hftp:proxy is specified, try http:proxy.
+	 if(hftp && !p)
+	    p=ResMgr::Query("http:proxy",c);
       }
       SetProxy(p);
    }
@@ -1668,7 +1736,7 @@ DirList *Http::MakeDirList(ArgV *args)
 {
    return new HttpDirList(args,this);
 }
-#include "Glob.h"
+#include "FileGlob.h"
 Glob *Http::MakeGlob(const char *pattern)
 {
    return new GenericGlob(this,pattern);
@@ -1899,9 +1967,9 @@ void Http::MakeSSLBuffers()
    Delete(send_buf);
    Delete(recv_buf);
 
-   SSL *ssl=lftp_ssl_new(sock);
-   IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT);
-   IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET);
+   SSL *ssl=lftp_ssl_new(sock,hostname);
+   IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT,hostname);
+   IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET,hostname);
    send_buf_ssl->DoConnect();
    recv_buf_ssl->CloseLater();
    send_buf=send_buf_ssl;
