@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.cc,v 1.40 2002/01/10 16:00:54 lav Exp $ */
+/* $Id: LocalAccess.cc,v 1.41 2002/04/18 10:37:22 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -564,7 +564,7 @@ int LocalListInfo::Do()
    return MOVED;
 }
 
-#include "Glob.h"
+#include "FileGlob.h"
 class LocalGlob : public Glob
 {
    const char *cwd;
