@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.82 2002/02/07 12:26:45 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.94 2002/09/02 11:05:30 lav Exp $ */
 
 #include <config.h>
 
@@ -40,19 +40,15 @@
 #include "url.h"
 #include "CopyJob.h"
 
-void  MirrorJob::PrintStatus(int v)
+void  MirrorJob::PrintStatus(int v,const char *tab)
 {
-   const char *tab="\t";
-
-   if(v==-1)
-      tab="";
-
    if(Done())
       goto final;
 
    switch(state)
    {
    case(INITIAL_STATE):
+   case(FINISHING):
    case(DONE):
    case(WAITING_FOR_TRANSFER):
    case(TARGET_REMOVE_OLD):
@@ -63,9 +59,12 @@ void  MirrorJob::PrintStatus(int v)
       printf("\tmkdir `%s' [%s]\n",target_dir,target_session->CurrentStatus());
       break;
 
-   case(CHANGING_DIR):
-      printf("\tcd `%s' [%s]\n",target_dir,target_session->CurrentStatus());
-      printf("\tcd `%s' [%s]\n",source_dir,source_session->CurrentStatus());
+   case(CHANGING_DIR_SOURCE):
+   case(CHANGING_DIR_TARGET):
+      if(target_session->IsOpen())
+	 printf("\tcd `%s' [%s]\n",target_dir,target_session->CurrentStatus());
+      if(source_session->IsOpen())
+	 printf("\tcd `%s' [%s]\n",source_dir,source_session->CurrentStatus());
       break;
 
    case(GETTING_LIST_INFO):
@@ -120,6 +119,7 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
    case(WAITING_FOR_TRANSFER):
    case(TARGET_REMOVE_OLD):
    case(TARGET_CHMOD):
+   case(FINISHING):
    case(DONE):
       Job::ShowRunStatus(s);
       break;
@@ -128,7 +128,8 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
       s->Show("mkdir `%s' [%s]",target_dir,target_session->CurrentStatus());
       break;
 
-   case(CHANGING_DIR):
+   case(CHANGING_DIR_SOURCE):
+   case(CHANGING_DIR_TARGET):
       if(target_session->IsOpen() && (!source_session->IsOpen() || now%4>=2))
 	 s->Show("cd `%s' [%s]",target_dir,target_session->CurrentStatus());
       else if(source_session->IsOpen())
@@ -203,9 +204,9 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 {
 	    if((flags&CONTINUE)
 	    && (old->defined&file->TYPE) && old->filetype==old->NORMAL
-	    && (file->defined&file->DATE)
-	    && (old->defined&old->DATE)
-	    && file->date + file->date_prec < old->date - old->date_prec
+	    && (flags&IGNORE_TIME ||
+	    	((file->defined&file->DATE) && (old->defined&old->DATE)
+	    	&& file->date + file->date_prec < old->date - old->date_prec))
 	    && (file->defined&file->SIZE) && (old->defined&old->SIZE)
 	    && file->size >= old->size)
 	    {
@@ -262,7 +263,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	    c->RemoveTargetFirst();
 	 CopyJob *cp=
 	    new CopyJob(c,file->name,"mirror");
-	 if(file->defined&file->DATE && file->date_prec<1)
+	 if(file->defined&file->DATE)
 	    cp->SetDate(file->date);
 	 if(file->defined&file->SIZE)
 	    cp->SetSize(file->size);
@@ -390,6 +391,8 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
       ignore|=FileInfo::IGNORE_SIZE_IF_OLDER|FileInfo::IGNORE_DATE_IF_OLDER;
    if(strcmp(target_session->GetProto(),"file"))
       ignore|=FileInfo::IGNORE_DATE_IF_OLDER;
+   if(flags&IGNORE_TIME)
+      ignore|=FileInfo::DATE;
    to_transfer->SubtractSame(dest,ignore);
 
    same->SubtractAny(to_transfer);
@@ -397,6 +400,9 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
    if(newer_than!=(time_t)-1)
       to_transfer->SubtractOlderThan(newer_than);
 
+   if(flags&NO_RECURSION)
+      to_transfer->SubtractDirs();
+
    new_files_set=new FileSet(to_transfer);
    new_files_set->SubtractAny(dest);
    old_files_set=new FileSet(dest);
@@ -445,7 +451,7 @@ void MirrorJob::HandleChdir(FileAccess * &session, int &redirections)
       eprintf("mirror: %s\n",session->StrError(res));
       stats.error_count++;
       transfer_count-=root_transfer_count;
-      state=DONE;
+      state=FINISHING;
       source_session->Close();
       target_session->Close();
       return;
@@ -455,13 +461,15 @@ void MirrorJob::HandleChdir(FileAccess * &session, int &redirections)
 }
 void MirrorJob::HandleListInfoCreation(FileAccess * &session,ListInfo * &list_info,const char *relative_dir)
 {
+   if(state!=GETTING_LIST_INFO)
+      return;
    list_info=session->MakeListInfo();
    if(list_info==0)
    {
       eprintf(_("mirror: protocol `%s' is not suitable for mirror\n"),
 	       session->GetProto());
       transfer_count-=root_transfer_count;
-      state=DONE;
+      state=FINISHING;
       return;
    }
    list_info->UseCache(use_cache);
@@ -484,7 +492,7 @@ void MirrorJob::HandleListInfo(ListInfo * &list_info, FileSet * &set)
       eprintf("mirror: %s\n",list_info->ErrorText());
       stats.error_count++;
       transfer_count-=root_transfer_count;
-      state=DONE;
+      state=FINISHING;
       Delete(source_list_info);
       source_list_info=0;
       Delete(target_list_info);
@@ -507,11 +515,26 @@ int   MirrorJob::Do()
    switch(state)
    {
    case(INITIAL_STATE):
+      source_session->Chdir(source_dir);
+      source_redirections=0;
+      Roll(source_session);
+      state=CHANGING_DIR_SOURCE;
+      m=MOVED;
+   case(CHANGING_DIR_SOURCE):
+      HandleChdir(source_session,source_redirections);
+      if(state!=CHANGING_DIR_SOURCE)
+	 return MOVED;
+      if(source_session->IsOpen())
+	 return m;
+
+      xfree(source_dir);
+      source_dir=xstrdup(source_session->GetCwd());
+
       if(!create_target_dir || !strcmp(target_dir,".") || !strcmp(target_dir,".."))
-	 goto pre_CHANGING_DIR;
+	 goto pre_CHANGING_DIR_TARGET;
       if(target_relative_dir)
 	 Report(_("Making directory `%s'"),target_relative_dir);
-      target_session->Mkdir(target_dir);
+      target_session->Mkdir(target_dir,(parent_mirror==0));
       state=MAKE_TARGET_DIR;
       m=MOVED;
    case(MAKE_TARGET_DIR):
@@ -520,38 +543,32 @@ int   MirrorJob::Do()
 	 return m;
       target_session->Close();
 
-   pre_CHANGING_DIR:
-      source_session->Chdir(source_dir);
+   pre_CHANGING_DIR_TARGET:
       target_session->Chdir(target_dir);
-      source_redirections=0;
       target_redirections=0;
-      Roll(source_session);
       Roll(target_session);
-      state=CHANGING_DIR;
+      state=CHANGING_DIR_TARGET;
       m=MOVED;
-   case(CHANGING_DIR):
-      HandleChdir(source_session,source_redirections);
+   case(CHANGING_DIR_TARGET):
       HandleChdir(target_session,target_redirections);
-      if(state!=CHANGING_DIR)
+      if(state!=CHANGING_DIR_TARGET)
 	 return MOVED;
-      if(source_session->IsOpen() || target_session->IsOpen())
+      if(target_session->IsOpen())
 	 return m;
 
       xfree(target_dir);
       target_dir=xstrdup(target_session->GetCwd());
-      xfree(source_dir);
-      source_dir=xstrdup(source_session->GetCwd());
 
+      state=GETTING_LIST_INFO;
+      m=MOVED;
       HandleListInfoCreation(source_session,source_list_info,source_relative_dir);
       HandleListInfoCreation(target_session,target_list_info,target_relative_dir);
-      if(state!=CHANGING_DIR)
+      if(state!=GETTING_LIST_INFO)
       {
 	 Delete(source_list_info); source_list_info=0;
 	 Delete(target_list_info); target_list_info=0;
 	 return MOVED;
       }
-      state=GETTING_LIST_INFO;
-      m=MOVED;
 
    case(GETTING_LIST_INFO):
       HandleListInfo(source_list_info,source_set);
@@ -654,7 +671,7 @@ int   MirrorJob::Do()
 
    pre_TARGET_CHMOD:
       if(flags&NO_PERMS)
-	 goto pre_DONE;
+	 goto pre_FINISHING;
 
       to_transfer->rewind();
       state=TARGET_CHMOD;
@@ -672,7 +689,7 @@ int   MirrorJob::Do()
       {
 	 file=to_transfer->curr();
 	 if(!file)
-	    goto pre_DONE;
+	    goto pre_FINISHING;
 	 to_transfer->next();
 	 if(!(file->defined&file->MODE))
 	    continue;
@@ -689,7 +706,7 @@ int   MirrorJob::Do()
       }
       break;
 
-   pre_DONE:
+   pre_FINISHING:
       if(target_is_local)     // FIXME
       {
 	 to_transfer->LocalUtime(target_dir,/*only_dirs=*/true);
@@ -700,21 +717,24 @@ int   MirrorJob::Do()
 	 if(flags&ALLOW_CHOWN)
 	    same->LocalChown(target_dir);
       }
-      if(waiting_num==0)
-	 transfer_count++; // parent mirror will decrement it.
-      state=DONE;
+      state=FINISHING;
       m=MOVED;
-   case(DONE):
+   case(FINISHING):
       j=FindDoneAwaitedJob();
       if(j)
       {
 	 RemoveWaiting(j);
 	 Delete(j);
 	 transfer_count--;
-	 if(waiting_num==0)
-	    transfer_count++; // parent mirror will decrement it.
 	 m=MOVED;
       }
+      if(waiting_num>0)
+	 break;
+
+      transfer_count++; // parent mirror will decrement it.
+      state=DONE;
+      m=MOVED;
+   case(DONE):
       break;
    }
    // give direct parent priority over grand-parents.
@@ -901,7 +921,7 @@ void MirrorJob::Statistics::Add(const Statistics &s)
 CMD(mirror)
 {
 #define args (parent->args)
-#define eprintf (parent->eprintf)
+#define eprintf parent->eprintf
    static struct option mirror_opts[]=
    {
       {"delete",no_argument,0,'e'},
@@ -923,6 +943,7 @@ CMD(mirror)
       {"use-cache",no_argument,0,256+'C'},
       {"Remove-source-files",no_argument,0,256+'R'},
       {"parallel",optional_argument,0,'P'},
+      {"ignore-time",no_argument,0,256+'i'},
       {0}
    };
 
@@ -1039,6 +1060,9 @@ CMD(mirror)
       case(256+'R'):
 	 remove_source_files=true;
 	 break;
+      case(256+'i'):
+	 flags|=MirrorJob::IGNORE_TIME;
+	 break;
       case('P'):
 	 if(optarg)
 	    parallel=atoi(optarg);
@@ -1101,6 +1125,17 @@ CMD(mirror)
 	    }
 	    target_dir=alloca_strdup(target_url.path);
 	 }
+	 if(target_dir[0] && target_dir[strlen(target_dir)-1]=='/')
+	 {
+	    // user wants source dir name appended.
+	    const char *base=basename_ptr(source_dir);
+	    if(base[0]!='/')
+	    {
+	       char *target_dir1=alloca_strdup2(target_dir,strlen(base));
+	       strcat(target_dir1,base);
+	       target_dir=target_dir1;
+	    }
+	 }
       }
       else
       {
