@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.52 2001/12/20 11:14:44 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.56 2002/07/18 09:13:48 lav Exp $ */
 
 #include <config.h>
 
@@ -269,7 +269,7 @@ void NetAccess::HandleTimeout()
 
 bool NetAccess::CheckTimeout()
 {
-   if(now >= event_time+timeout)
+   if(time_t(now) >= event_time+timeout)
    {
       HandleTimeout();
       return(true);
@@ -333,12 +333,6 @@ int NetAccess::Resolve(const char *defp,const char *ser,const char *pr)
    if(resolver->Error())
    {
       SetError(LOOKUP_ERROR,resolver->ErrorMsg());
-      xfree(hostname);
-      hostname=0;
-      xfree(portname);
-      portname=0;
-      xfree(cwd);
-      cwd=0;
       return(MOVED);
    }
 
@@ -357,13 +351,16 @@ int NetAccess::Resolve(const char *defp,const char *ser,const char *pr)
 
 // RateLimit class implementation.
 int RateLimit::total_xfer_number;
-RateLimit::BytesPool RateLimit::total;
+RateLimit::BytesPool RateLimit::total[2];
 bool RateLimit::total_reconfig_needed=true;
 
 RateLimit::RateLimit(const char *c)
 {
    if(total_xfer_number==0)
-      total.Reset();
+   {
+      total[GET].Reset();
+      total[PUT].Reset();
+   }
    total_xfer_number++;
    Reconfig(0,c);
 }
@@ -395,22 +392,22 @@ void RateLimit::BytesPool::AdjustTime()
    }
 }
 
-int RateLimit::BytesAllowed()
+int RateLimit::BytesAllowed(dir_t dir)
 {
    if(total_reconfig_needed)
       ReconfigTotal();
 
-   if(one.rate==0 && total.rate==0) // unlimited
+   if(one[dir].rate==0 && total[dir].rate==0) // unlimited
       return LARGE;
 
-   one  .AdjustTime();
-   total.AdjustTime();
+   one  [dir].AdjustTime();
+   total[dir].AdjustTime();
 
    int ret=LARGE;
-   if(total.rate>0)
-      ret=total.pool/total_xfer_number;
-   if(one.rate>0 && ret>one.pool)
-      ret=one.pool;
+   if(total[dir].rate>0)
+      ret=total[dir].pool/total_xfer_number;
+   if(one[dir].rate>0 && ret>one[dir].pool)
+      ret=one[dir].pool;
    return ret;
 }
 
@@ -422,10 +419,10 @@ void RateLimit::BytesPool::Used(int bytes)
       pool-=bytes;
 }
 
-void RateLimit::BytesUsed(int bytes)
+void RateLimit::BytesUsed(int bytes,dir_t dir)
 {
-   total.Used(bytes);
-   one  .Used(bytes);
+   total[dir].Used(bytes);
+   one  [dir].Used(bytes);
 }
 
 void RateLimit::BytesPool::Reset()
@@ -437,18 +434,20 @@ void RateLimit::Reconfig(const char *name,const char *c)
 {
    if(name && strncmp(name,"net:limit-",10))
       return;
-   one.rate     = ResMgr::Query("net:limit-rate",c);
-   one.pool_max = ResMgr::Query("net:limit-max",c);
-   one.Reset(); // to cut bytes_pool.
+   ResMgr::Query("net:limit-rate",c).ToNumberPair(one[GET].rate,one[PUT].rate);
+   ResMgr::Query("net:limit-max",c) .ToNumberPair(one[GET].pool_max,one[PUT].pool_max);
+   one[GET].Reset(); // to cut bytes_pool.
+   one[PUT].Reset();
 
    if(name && !strncmp(name,"net:limit-total-",16))
       total_reconfig_needed=true;
 }
 void RateLimit::ReconfigTotal()
 {
-   total.rate     = ResMgr::Query("net:limit-total-rate",0);
-   total.pool_max = ResMgr::Query("net:limit-total-max",0);
-   total.Reset();
+   ResMgr::Query("net:limit-total-rate",0).ToNumberPair(total[GET].rate,total[PUT].rate);
+   ResMgr::Query("net:limit-total-max",0) .ToNumberPair(total[GET].pool_max,total[PUT].pool_max);
+   total[GET].Reset();
+   total[PUT].Reset();
    total_reconfig_needed = false;
 }
 
@@ -480,7 +479,7 @@ bool NetAccess::ReconnectAllowed()
    if(try_time==0)
       return true;
    long interval = ReconnectInterval();
-   if(now >= try_time+interval)
+   if(time_t(now) >= try_time+interval)
       return true;
    TimeoutS(interval-(time_t(now)-try_time));
    return false;
