@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.24 2001/11/06 13:15:33 lav Exp $ */
+/* $Id: NetAccess.h,v 1.29 2002/07/18 09:13:49 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -53,7 +53,6 @@ protected:
 
    int	 connection_limit;
    bool	 connection_takeover;
-   time_t takeover_time;
 
    int	 socket_buffer;
    int	 socket_maxseg;
@@ -130,59 +129,42 @@ private:
    static int total_xfer_number;
    static bool total_reconfig_needed;
    static void ReconfigTotal();
-   static BytesPool total;
-   BytesPool one;
+   static BytesPool total[2];
+   BytesPool one[2];
 
 public:
    RateLimit(const char *closure);
    ~RateLimit();
 
-   int BytesAllowed();
-   void BytesUsed(int);
+   enum dir_t { GET=0, PUT=1 };
+
+   int BytesAllowed(dir_t how);
+   int BytesAllowedToGet() { return BytesAllowed(GET); }
+   int BytesAllowedToPut() { return BytesAllowed(PUT); }
+   void BytesUsed(int,dir_t);
+   void BytesGot(int b) { BytesUsed(b,GET); }
+   void BytesPut(int b) { BytesUsed(b,PUT); }
 
    void Reconfig(const char *name,const char *c);
 };
 
-class GenericParseGlob : public Glob
-{
-protected:
-   FileAccess *session;
-   char	 *dir;
-   const char *curr_dir;
-   FileSet *dir_list;
-   int   dir_index;
-   GenericParseGlob *updir_glob;
-
-   Buffer *ubuf;
-
-   virtual FileSet *Parse(const char *buf,int len)=0;
-   virtual GenericParseGlob *MakeUpDirGlob()=0;
-
-public:
-   int	 Do();
-   const char *Status();
-
-   GenericParseGlob(FileAccess *s,const char *n_pattern);
-   virtual ~GenericParseGlob();
-};
-
 class GenericParseListInfo : public ListInfo
 {
 protected:
-   FileAccess *session;
-
    FA::fileinfo *get_info;
    int get_info_cnt;
 
+   int mode;
    Buffer *ubuf;
 
    bool get_time_for_dirs;
    bool can_get_prec_time;
 
-   virtual FileSet *Parse(const char *buf,int len)=0;
+   virtual FileSet *Parse(const char *buf,int len)
+      { return session->ParseLongList(buf,len); }
 
 public:
-   GenericParseListInfo(FileAccess *session);
+   GenericParseListInfo(FileAccess *session,const char *path);
    virtual ~GenericParseListInfo();
    int Do();
    const char *Status();
