@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.39 2002/02/07 12:26:45 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.45 2002/07/24 11:09:56 lav Exp $ */
 
 #include <config.h>
 
@@ -27,9 +27,11 @@
 #include <unistd.h>
 #include <math.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <errno.h>
 extern "C" {
 #include <regex.h>
 }
@@ -37,6 +39,7 @@ extern "C" {
 #include "SMTask.h"
 #include "xmalloc.h"
 #include "xalloca.h"
+#include "misc.h"
 
 ResMgr::Resource  *ResMgr::chain=0;
 ResDecl		  *ResMgr::type_chain=0;
@@ -547,7 +550,7 @@ ResValue ResMgr::Query(const char *name,const char *closure)
    if(msg)
    {
       // debug only
-      fprintf(stderr,_("Query of variable `%s' failed: %s\n"),name,msg);
+      // fprintf(stderr,_("Query of variable `%s' failed: %s\n"),name,msg);
       return 0;
    }
 
@@ -782,7 +785,79 @@ const char *ResMgr::IPv4AddrValidate(char **value)
    return 0;
 }
 
+const char *ResMgr::FileReadable(char **value)
+{
+   if(!**value)
+      return 0;
+   const char *f=expand_home_relative(*value);
+   char *cwd=0;
+   const char *error=0;
+   if(f[0]!='/')
+   {
+      cwd=xgetcwd();
+      if(cwd)
+	 f=dir_file(cwd,f);
+   }
+   if(access(f,R_OK)<0)
+      error=strerror(errno);
+   else
+   {
+      xfree(*value);
+      *value=xstrdup(f);
+   }
+   xfree(cwd);
+   return error;
+}
+
+const char *ResMgr::DirReadable(char **value)
+{
+   if(!**value)
+      return 0;
+   const char *f=expand_home_relative(*value);
+   char *cwd=0;
+   const char *error=0;
+   if(f[0]!='/')
+   {
+      cwd=xgetcwd();
+      if(cwd)
+	 f=dir_file(cwd,f);
+   }
+   struct stat st;
+   if(stat(f,&st)<0)
+      error=strerror(errno);
+   else if(!S_ISDIR(st.st_mode))
+      error=strerror(ENOTDIR);
+   else if(access(f,R_OK|X_OK)<0)
+      error=strerror(errno);
+   else
+   {
+      xfree(*value);
+      *value=xstrdup(f);
+   }
+   xfree(cwd);
+   return 0;
+}
+
 const char *ResMgr::NoClosure(char **)
 {
    return _("no closure defined for this setting");
 }
+
+const char *ResMgr::UNumberPairValidate(char **value)
+{
+   char *v=*value;
+
+   int n=0;
+
+   unsigned a,b;
+   if(2>sscanf(v,"%u%*c%u%n",&a,&b,&n))
+   {
+      if(UNumberValidate(value))
+	 return _("invalid pair of numbers");
+      return 0;
+   }
+
+   v[n]=0;
+
+   return 0;
+}
