@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.176 2002/02/11 16:56:05 lav Exp $ */
+/* $Id: commands.cc,v 1.187 2002/09/02 11:05:31 lav Exp $ */
 
 #include <config.h>
 
@@ -49,6 +49,8 @@
 #include "FindJobDu.h"
 #include "ChmodJob.h"
 #include "CopyJob.h"
+#include "OutputJob.h"
+#include "echoJob.h"
 
 #include "misc.h"
 #include "alias.h"
@@ -69,6 +71,7 @@
 #include "FileSetOutput.h"
 #include "PatternSet.h"
 #include "LocalDir.h"
+#include "ConnectionSlot.h"
 
 #include "confpaths.h"
 
@@ -88,7 +91,7 @@ CMD(wait);  CMD(subsh);  CMD(mirror);
 CMD(mv);    CMD(cat);    CMD(cache);
 CMD(mkdir); CMD(scache); CMD(mrm);
 CMD(ver);   CMD(close);  CMD(bookmark);CMD(lftp);
-CMD(echo);  CMD(suspend);CMD(sleep);
+CMD(echo);  CMD(suspend);CMD(sleep);   CMD(slot);
 CMD(at);    CMD(find);   CMD(command); CMD(module);
 CMD(lpwd);  CMD(glob);	 CMD(chmod);   CMD(queue);
 CMD(repeat);CMD(get1);   CMD(history);
@@ -307,6 +310,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -e, --delete           delete files not present at remote site\n"
 	 " -s, --allow-suid       set suid/sgid bits according to remote site\n"
 	 "     --allow-chown      try to set owner and group on files\n"
+	 "     --ignore-time      ignore time when deciding whether to download\n"
 	 " -n, --only-newer       download only newer files (-c won't work)\n"
 	 " -r, --no-recursion     don't go to subdirectories\n"
 	 " -p, --no-perms         don't set file permissions\n"
@@ -444,6 +448,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "Sleep for given amount of time. The time argument can be optionally\n"
 	 "followed by unit specifier: d - days, h - hours, m - minutes, s - seconds.\n"
 	 "By default time is assumed to be seconds.\n")},
+   {"slot",    cmd_slot,   0,0},
    {"source",  cmd_source, N_("source <file>"),
 	 N_("Execute commands recorded in file <file>\n")},
    {"suspend", cmd_suspend},
@@ -589,6 +594,8 @@ Job *CmdExec::builtin_cd()
    if (!verify_path || background)
    {
       session->Chdir(dir,false);
+      if(slot)
+	 ConnectionSlot::SetCwd(slot,session->GetCwd());
       exit_code=0;
       return 0;
    }
@@ -849,7 +856,7 @@ Job *CmdExec::builtin_open()
       {
 	 ChangeSession(new DummyProto);
       }
-      if(host && host[0])
+      if(host && host[0] && session->GetHostName()==0)
 	 session->Connect(host,port);
       if(user)
       {
@@ -884,7 +891,7 @@ Job *CmdExec::builtin_open()
       if(old)
 	 session->Chdir(old,false);
 
-      char *s=string_alloca(strlen(path)*4+40);
+      char *s=string_alloca(strlen(path)*2+40);
       strcpy(s,"&& cd \"");
       unquote(s+strlen(s),path);
       strcat(s,"\"");
@@ -900,6 +907,9 @@ Job *CmdExec::builtin_open()
    if(url)
       delete url;
 
+   if(slot)
+      ConnectionSlot::Set(slot,session);
+
    Reconfig(0);
 
    if(builtin==BUILTIN_OPEN)
@@ -1000,7 +1010,11 @@ Job *CmdExec::builtin_queue()
       switch(opt)
       {
       case 'n':
-	 if(!isdigit((unsigned char)optarg[0]))
+	 /* Actually, sending pos == -1 will work, but it'll put the
+	  * job at the end; it's confusing for "-n 0" to mean "put
+	  * it at the end", and that's the default anyway, so disallow
+	  * it. */
+	 if(!isdigit((unsigned char)optarg[0]) || atoi(optarg) == 0)
 	 {
 	    eprintf(_("%s: -n: Number expected. "), args->a0());
 	    goto err;
@@ -1048,18 +1062,37 @@ Job *CmdExec::builtin_queue()
    const int args_remaining = args->count() - args->getindex();
    switch(mode) {
       case ins: {
+	 CmdExec *queue=GetQueue(false);
 	 if(args_remaining==0)
-	    goto err;
-
-	 CmdExec *queue=GetQueue();
+	 {
+	    if(!queue)
+	    {
+	       if(verbose)
+		  printf(_("Created a stopped queue.\n"));
+	       queue=GetQueue(true);
+	       queue->Suspend();
+	    }
+	    else
+	       queue->PrintStatus(0,"");
+	    exit_code=0;
+	    break;
+	 }
+	 if(!queue)
+	    queue=GetQueue(true);
 
 	 char *cmd=NULL;
 	 if(args_remaining == 1)
-		 cmd=args->Combine(args->getindex());
+	    cmd=args->Combine(args->getindex());
 	 else
-		 cmd=args->CombineQuoted(args->getindex());
+	    cmd=args->CombineQuoted(args->getindex());
 
-	 queue->has_queue->QueueCmd(cmd, session->GetCwd(), cwd?cwd->GetName():0, pos, verbose);
+	 if(!strcasecmp(cmd,"stop"))
+	    queue->Suspend();
+	 else if(!strcasecmp(cmd,"start"))
+	    queue->Resume();
+	 else
+	    queue->queue_feeder->QueueCmd(cmd, session->GetCwd(),
+					  cwd?cwd->GetName():0, pos, verbose);
 	 xfree(cmd);
 
 	 last_bg=queue->jobno;
@@ -1084,11 +1117,11 @@ Job *CmdExec::builtin_queue()
 	 }
 
 	 if(!arg)
-	    queue->has_queue->DelJob(-1, verbose); /* delete the last job */
-	 else if(isdigit(arg[0]))
-	    queue->has_queue->DelJob(atoi(arg)-1, verbose);
+	    queue->queue_feeder->DelJob(-1, verbose); /* delete the last job */
+	 else if(isdigit(arg[0]) && atoi(arg) != 0)
+	    queue->queue_feeder->DelJob(atoi(arg)-1, verbose);
 	 else
-	    queue->has_queue->DelJob(arg, verbose);
+	    queue->queue_feeder->DelJob(arg, verbose);
 
 	 exit_code=0;
       }
@@ -1113,12 +1146,12 @@ Job *CmdExec::builtin_queue()
 	    break;
 	 }
 
-	 if(isdigit(arg[0])) {
-	    queue->has_queue->MoveJob(atoi(arg)-1, to, verbose);
+	 if(isdigit(arg[0]) && atoi(arg) != 0) {
+	    queue->queue_feeder->MoveJob(atoi(arg)-1, to, verbose);
 	    break;
 	 }
 
-	 queue->has_queue->MoveJob(arg, to, verbose);
+	 queue->queue_feeder->MoveJob(arg, to, verbose);
 	 exit_code=0;
       }
       break;
@@ -1218,8 +1251,9 @@ CMD(ls)
 
    char *a=args->Combine(nlist?1:0);
 
-   if(!nlist && args->count()==1 && parent->var_ls && parent->var_ls[0])
-      args->Append(parent->var_ls);
+   const char *var_ls=ResMgr::Query("cmd:ls-default",session->GetConnectURL(FA::NO_PATH));
+   if(!nlist && args->count()==1 && var_ls[0])
+      args->Append(var_ls);
 
    FileCopyPeer *src_peer=0;
    if(!nlist)
@@ -1383,11 +1417,11 @@ CMD(cls)
    const char *op=args->a0();
    bool re=false;
 
-   if(!output)
-      output=new FDStream(1,"<stdout>");
+   OutputJob *out=new OutputJob(output, args->a0());
+   output=0;
 
    FileSetOutput fso;
-   fso.config(output);
+   fso.config(out);
 
    if(!strncmp(op,"re",2))
       re=true;
@@ -1398,34 +1432,15 @@ CMD(cls)
    if(const char *err = fso.parse_argv(args)) {
       eprintf("%s: %s.\n", op, err);
       eprintf(_("Try `help %s' for more information.\n"),op);
+      delete out;
       return 0;
    }
 
-   char *a=args->Combine(0);
-
-   FileCopyPeer *src_peer=0;
-   src_peer=new FileCopyPeerCLS(session->Clone(),args,fso);
-   args=0;
-
+   clsJob *j = new clsJob(session->Clone(), args, fso, out);
    if(re)
-      src_peer->NoCache();
-   src_peer->SetDate(NO_DATE);
-   src_peer->SetSize(NO_SIZE);
-   FileCopyPeer *dst_peer=new FileCopyPeerFDStream(output,FileCopyPeer::PUT);
+      j->UseCache(false);
 
-   FileCopy *c=FileCopy::New(src_peer,dst_peer,false);
-   c->DontCopyDate();
-   c->LineBuffered();
-   c->Ascii();
-
-   CopyJob *j=new CopyJob(c,a,op);
-   if(fso.quiet)
-      j->NoStatus();
-   else if(output->usesfd(1))
-      j->NoStatusOnWrite();
-
-   xfree(a);
-   output=0;
+   args=0;
 
    return j;
 }
@@ -1462,7 +1477,8 @@ CMD(cat)
       eprintf(_("Usage: %s [OPTS] files...\n"),op);
       return 0;
    }
-   CatJob *j=new CatJob(session->Clone(),output,args);
+   OutputJob *out=new OutputJob(output, args->a0());
+   CatJob *j=new CatJob(session->Clone(),out,args);
    if(!auto_ascii)
    {
       if(ascii)
@@ -1765,8 +1781,11 @@ CMD(pwd)
    char *url=alloca_strdup(url_c);
    int len=strlen(url_c);
    url[len++]='\n';  // replaces \0
-   Job *j=CopyJob::NewEcho(url,len,output,args->a0());
+
+   OutputJob *out=new OutputJob(output, args->a0());
+   Job *j=new echoJob(url,len,out);
    output=0;
+
    return j;
 }
 
@@ -2006,7 +2025,8 @@ CMD(set)
    if(a==0)
    {
       char *s=ResMgr::Format(with_defaults,only_defaults);
-      Job *j=CopyJob::NewEcho(s,output,args->a0());
+      OutputJob *out=new OutputJob(output, args->a0());
+      Job *j=new echoJob(s,out);
       xfree(s);
       output=0;
       return j;
@@ -2050,7 +2070,8 @@ CMD(alias)
    if(args->count()<2)
    {
       char *list=Alias::Format();
-      Job *j=CopyJob::NewEcho(list,output,args->a0());
+      OutputJob *out=new OutputJob(output, args->a0());
+      Job *j=new echoJob(list,out);
       xfree(list);
       output=0;
       return j;
@@ -2185,9 +2206,9 @@ CMD(cache)  // cache control
    else if(!strcasecmp(op,"flush"))
       LsCache::Flush();
    else if(!strcasecmp(op,"on"))
-      LsCache::On();
+      ResMgr::Set("cache:enable",0,"yes");
    else if(!strcasecmp(op,"off"))
-      LsCache::Off();
+      ResMgr::Set("cache:enable",0,"no");
    else if(!strcasecmp(op,"size"))
    {
       op=args->getnext();
@@ -2197,14 +2218,13 @@ CMD(cache)  // cache control
 	 exit_code=1;
 	 return 0;
       }
-      long lim=-1;
-      if(strcasecmp(op,"unlim") && sscanf(op,"%ld",&lim)!=1)
+      const char *err=ResMgr::Set("cache:size",0,op);
+      if(err)
       {
-	 eprintf(_("%s: Invalid number for size\n"),args->a0());
+	 eprintf("%s: %s: %s\n",args->a0(),op,err);
 	 exit_code=1;
 	 return 0;
       }
-      LsCache::SetSizeLimit(lim);
    }
    else if(!strcasecmp(op,"expire"))
    {
@@ -2215,14 +2235,13 @@ CMD(cache)  // cache control
 	 exit_code=1;
 	 return 0;
       }
-      TimeInterval exp(op);
-      if(exp.Error())
+      const char *err=ResMgr::Set("cache:expire",0,op);
+      if(err)
       {
-	 eprintf("%s: %s: %s.\n",args->a0(),op,exp.ErrorText());
+	 eprintf("%s: %s: %s\n",args->a0(),op,err);
 	 exit_code=1;
 	 return 0;
       }
-      LsCache::SetExpire(exp);
    }
    return 0;
 }
@@ -2401,7 +2420,8 @@ CMD(bookmark)
    if(!strcasecmp(op,"list"))
    {
       char *list=lftp_bookmarks.FormatHidePasswords();
-      Job *j=CopyJob::NewEcho(list,output,args->a0());
+      OutputJob *out=new OutputJob(output, args->a0());
+      Job *j=new echoJob(list,out);
       xfree(list);
       output=0;
       return j;
@@ -2490,7 +2510,10 @@ CMD(echo)
    {
       s[len++]='\n'; // replaces \0 char
    }
-   Job *j=CopyJob::NewEcho(s,len,output,args->a0());
+
+   OutputJob *out=new OutputJob(output, args->a0());
+   Job *j=new echoJob(s,len,out);
+
    xfree(s);
    output=0;
    return j;
@@ -2537,6 +2560,7 @@ CMD(find)
    FinderJob_List *j=new class FinderJob_List(session->Clone(),args,
       output?output:new FDStream(1,"<stdout>"));
    j->set_maxdepth(maxdepth);
+   args=0;
    output=0;
    return j;
 }
@@ -2739,7 +2763,7 @@ CMD(lpwd)
    const char *name=parent->cwd->GetName();
    char *buf=alloca_strdup2(name,2);
    sprintf(buf,"%s\n",name?name:"?");
-   Job *j=CopyJob::NewEcho(buf,strlen(buf),output,args->a0());
+   Job *j=new echoJob(buf,new OutputJob(output, args->a0()));
    output=0;
    return j;
 }
@@ -3020,3 +3044,21 @@ CMD(history)
 
    return 0;
 }
+
+CMD(slot)
+{
+   const char *n=args->getarg(1);
+   if(n)
+   {
+      parent->ChangeSlot(n);
+      return 0;
+   }
+   else
+   {
+      char *slots=ConnectionSlot::Format();
+      Job *j=new echoJob(slots,new OutputJob(output,args->a0()));
+      xfree(slots);
+      output=0;
+      return j;
+   }
+}
