@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.cc,v 1.241 2002/03/26 12:38:56 lav Exp $ */
+/* $Id: ftpclass.cc,v 1.251 2002/07/24 11:09:57 lav Exp $ */
 
 #include <config.h>
 
@@ -36,7 +36,7 @@
 #include "xalloca.h"
 #include "url.h"
 #include "FtpListInfo.h"
-#include "Glob.h"
+#include "FileGlob.h"
 #include "FtpDirList.h"
 #include "log.h"
 #include "FileCopyFtp.h"
@@ -322,8 +322,7 @@ void Ftp::NoFileCheck(int act)
 /* 5xx that aren't errors at all */
 bool Ftp::NonError5XX(int act)
 {
-   /* don't strstr; we might have done "NLST No files found" */
-   return (mode==LIST && act==RESP_NO_FILE && !strcmp(line,"550 No files found."));
+   return (mode==LIST && act==RESP_NO_FILE && (!file || !file[0]));
 }
 
 /* 5xx that are really transient like 4xx */
@@ -705,10 +704,11 @@ int Ftp::Handle_PASV()
 
    if((a0==0 && a1==0 && a2==0 && a3==0)
    || (QueryBool("fix-pasv-address",hostname)
-       && InPrivateNetwork(&data_sa) && !InPrivateNetwork(&peer_sa)))
+       && (InPrivateNetwork(&data_sa) != InPrivateNetwork(&peer_sa))))
    {
       // broken server, try to fix up
       fixed_pasv=true;
+      DebugPrint("---- ","Address returned by PASV seemed to be incorrect and has been fixed",2);
       if(data_sa.sa.sa_family==AF_INET)
 	 memcpy(a,&peer_sa.in.sin_addr,sizeof(peer_sa.in.sin_addr));
 #if INET6
@@ -1245,12 +1245,12 @@ int   Ftp::Do()
 	 goto usual_return;
 
       addr_len=sizeof(peer_sa);
-      getsockname(control_sock,&peer_sa.sa,&addr_len);
+      getpeername(control_sock,&peer_sa.sa,&addr_len);
 
 #ifdef USE_SSL
       if(proxy?!strncmp(proxy,"ftps://",7):ftps)
       {
-	 control_ssl=lftp_ssl_new(control_sock);
+	 control_ssl=lftp_ssl_new(control_sock,hostname);
 	 control_ssl_connected=false;
 	 prot='P';
       }
@@ -1287,18 +1287,7 @@ int   Ftp::Do()
 #ifdef USE_SSL
       if(auth_tls_sent && !RespQueueIsEmpty())
 	 goto usual_return;
-      if(control_ssl)
-      {
-	 SendCmd("PBSZ 0");
-	 AddResp(0,CHECK_IGNORE);
-	 const char *want_prot=QueryBool("ssl-protect-data",hostname)?"P":"C";
-	 if(*want_prot!=prot)
-	 {
-	    SendCmd2("PROT",want_prot);
-	    AddResp(200,CHECK_PROT);
-	 }
-      }
-#endif // USE_SSL
+#endif
 
       char *user_to_use=(user?user:anon_user);
       if(proxy)
@@ -1361,6 +1350,20 @@ int   Ftp::Do()
 	 AddResp(RESP_PWD_MKD_OK,CHECK_PWD);
       }
 
+#ifdef USE_SSL
+      if(control_ssl)
+      {
+	 SendCmd("PBSZ 0");
+	 AddResp(0,CHECK_IGNORE);
+	 const char *want_prot=QueryBool("ssl-protect-data",hostname)?"P":"C";
+	 if(*want_prot!=prot)
+	 {
+	    SendCmd2("PROT",want_prot);
+	    AddResp(200,CHECK_PROT);
+	 }
+      }
+#endif // USE_SSL
+
       set_real_cwd(0);
       type=FTP_TYPE_A;	   // just after login we are in TEXT mode
       state=EOF_STATE;
@@ -1924,7 +1927,7 @@ int   Ftp::Do()
 #ifdef USE_SSL
       if(prot=='P')
       {
-	 data_ssl=lftp_ssl_new(data_sock);
+	 data_ssl=lftp_ssl_new(data_sock,hostname);
 	 // share session id between control and data connections.
 	 SSL_copy_session_id(data_ssl,control_ssl);
 	 data_ssl_connected=false;
@@ -2057,7 +2060,8 @@ notimeout_return:
       else if(state==DATA_OPEN_STATE)
       {
 	 assert(rate_limit!=0);
-	 int bytes_allowed = rate_limit->BytesAllowed();
+	 int bytes_allowed = rate_limit->BytesAllowed(
+			      mode==STORE?rate_limit->PUT:rate_limit->GET);
 	 // guard against unimplemented REST: if we have sent REST command
 	 // (real_pos==-1) and did not yet receive the response
 	 // (RespQueueSize()>1), don't allow to read/write the data
@@ -2313,7 +2317,8 @@ int  Ftp::ReceiveResp()
 	    {
 	       if(!control_ssl_connected)
 	       {
-		  res=SSL_connect(control_ssl);
+		  errno=0;
+		  res=lftp_ssl_connect(control_ssl,hostname);
 		  if(res<=0)
 		  {
 		     if(BIO_sock_should_retry(res))
@@ -2325,12 +2330,16 @@ int  Ftp::ReceiveResp()
 			return m;
 		     else // error
 		     {
-			SetError(FATAL,lftp_ssl_strerror("SSL connect"));
+			if(errno && TemporaryNetworkError(errno))
+			   Disconnect();
+			else
+			   SetError(FATAL,lftp_ssl_strerror("SSL connect"));
 			return MOVED;
 		     }
 		  }
 		  control_ssl_connected=true;
 	       }
+	       errno=0;
 	       res=SSL_read(control_ssl,resp+resp_size,resp_alloc-resp_size-1);
 	       if(res<0)
 	       {
@@ -2368,7 +2377,7 @@ int  Ftp::ReceiveResp()
 	    if(res==0)
 	    {
 	       DebugPrint("**** ",_("Peer closed connection"),0);
-	       ControlClose();
+	       quit_sent=true;
 	       Disconnect();
 	       return MOVED;
 	    }
@@ -2407,12 +2416,26 @@ int  Ftp::ReceiveResp()
 
 void Ftp::SendUrgentCmd(const char *cmd)
 {
-   assert(!control_ssl);   // no way to send urgent data over ssl
-   FlushSendQueue(/*all=*/true);
    static const char pre_cmd[]={TELNET_IAC,TELNET_IP,TELNET_IAC,TELNET_SYNCH};
-   /* send only first byte as OOB due to OOB braindamage in many unices */
-   send(control_sock,pre_cmd,1,MSG_OOB);
-   send(control_sock,pre_cmd+1,sizeof(pre_cmd)-1,0);
+
+   int fl=fcntl(control_sock,F_GETFL);
+   fcntl(control_sock,F_SETFL,fl&~O_NONBLOCK);
+   FlushSendQueue(/*all=*/true);
+#ifdef USE_SSL
+   if(control_ssl)
+   {
+      // no way to send urgent data over ssl, send normally.
+      SSL_write(control_ssl,pre_cmd,4);
+   }
+   else
+#endif
+   {
+      /* send only first byte as OOB due to OOB braindamage in many unices */
+      send(control_sock,pre_cmd,1,MSG_OOB);
+      send(control_sock,pre_cmd+1,sizeof(pre_cmd)-1,0);
+   }
+   fcntl(control_sock,F_SETFL,fl);
+
    SendCmd(cmd);
 }
 
@@ -2446,7 +2469,7 @@ void  Ftp::DataAbort()
 
    CloseRespQueue();
 
-   if(!QueryBool("use-abor",hostname) || control_ssl
+   if(!QueryBool("use-abor",hostname)
    || RespQueueSize()>1)
    {
       if(copy_mode==COPY_NONE
@@ -2520,6 +2543,8 @@ void  Ftp::Disconnect()
       return;
    disconnect_in_progress=true;
 
+   bool no_greeting=(!RespQueueIsEmpty() && RespQueue[RQ_head].check_case==CHECK_READY);
+
    DataAbort();
    DataClose();
    if(control_sock>=0 && state!=CONNECTING_STATE && !quit_sent
@@ -2533,7 +2558,7 @@ void  Ftp::Disconnect()
    ControlClose();
    AbortedClose();
 
-   if(state==CONNECTING_STATE)
+   if(state==CONNECTING_STATE || no_greeting)
       NextPeer();
 
    if(copy_mode!=COPY_NONE)
@@ -2616,7 +2641,8 @@ int  Ftp::FlushSendQueue(bool all)
       {
 	 if(!control_ssl_connected)
 	 {
-	    res=SSL_connect(control_ssl);
+	    errno=0;
+	    res=lftp_ssl_connect(control_ssl,hostname);
 	    if(res<=0)
 	    {
 	       if(BIO_sock_should_retry(res))
@@ -2628,7 +2654,10 @@ int  Ftp::FlushSendQueue(bool all)
 		  return m;
 	       else // error
 	       {
-		  SetError(FATAL,lftp_ssl_strerror("SSL connect"));
+		  if(errno && TemporaryNetworkError(errno))
+		     Disconnect();
+		  else
+		     SetError(FATAL,lftp_ssl_strerror("SSL connect"));
 		  return MOVED;
 	       }
 	    }
@@ -2920,7 +2949,6 @@ int   Ftp::Read(void *buf,int size)
    int res,shift;
 
    Resume();
-   Do();
    if(Error())
       return(error_code);
 
@@ -2969,7 +2997,7 @@ read_again:
 
    {
       assert(rate_limit!=0);
-      int allowed=rate_limit->BytesAllowed();
+      int allowed=rate_limit->BytesAllowedToGet();
       if(allowed==0)
 	 return DO_AGAIN;
       if(size>allowed)
@@ -2983,7 +3011,7 @@ read_again:
       if(!data_ssl_connected)
       {
 	 errno=0;
-	 res=SSL_connect(data_ssl);
+	 res=lftp_ssl_connect(data_ssl,hostname);
 	 if(res<=0)
 	 {
 	    if(BIO_sock_should_retry(res))
@@ -3065,7 +3093,7 @@ read_again:
    retries=0;
    persist_retries=0;
    assert(rate_limit!=0);
-   rate_limit->BytesUsed(res);
+   rate_limit->BytesGot(res);
    real_pos+=res;
    if(real_pos<=pos)
       goto read_again;
@@ -3096,7 +3124,6 @@ int   Ftp::Write(const void *buf,int size)
       return(0);
 
    Resume();
-   Do();
    if(Error())
       return(error_code);
 
@@ -3105,7 +3132,7 @@ int   Ftp::Write(const void *buf,int size)
 
    {
       assert(rate_limit!=0);
-      int allowed=rate_limit->BytesAllowed();
+      int allowed=rate_limit->BytesAllowedToPut();
       if(allowed==0)
 	 return DO_AGAIN;
       if(size>allowed)
@@ -3119,7 +3146,7 @@ int   Ftp::Write(const void *buf,int size)
       if(!data_ssl_connected)
       {
 	 errno=0;
-	 res=SSL_connect(data_ssl);
+	 res=lftp_ssl_connect(data_ssl,hostname);
 	 if(res<0)
 	 {
 	    if(BIO_sock_should_retry(res))
@@ -3186,7 +3213,7 @@ int   Ftp::Write(const void *buf,int size)
    retries=0;
    persist_retries=0;
    assert(rate_limit!=0);
-   rate_limit->BytesUsed(res);
+   rate_limit->BytesPut(res);
    pos+=res;
    real_pos+=res;
    flags|=IO_FLAG;
@@ -3644,7 +3671,9 @@ const char *Ftp::CurrentStatus()
 	 return(_("Waiting for response..."));
       return(_("Making data connection..."));
    case(CWD_CWD_WAITING_STATE):
-      return(_("Changing remote directory..."));
+      if(FindLastCWD())
+	 return(_("Changing remote directory..."));
+      /*fallthrough*/
    case(WAITING_STATE):
       if(copy_mode==COPY_SOURCE)
 	 return "";
