@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.42.2.1 2002/01/15 16:20:26 lav Exp $ */
+/* $Id: misc.cc,v 1.56 2002/08/05 08:51:14 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -46,6 +46,11 @@
 #include "ProcWait.h"
 #include "SignalHook.h"
 #include "url.h"
+#include "ResMgr.h"
+
+#if defined(HAVE_UNSETENV) && !defined(HAVE_UNSETENV_DECL)
+CDECL void unsetenv(const char *name);
+#endif
 
 const char *dir_file(const char *dir,const char *file)
 {
@@ -97,14 +102,11 @@ const char *url_file(const char *url,const char *file)
       strcpy(buf,dir_file(url,file));
       return buf;
    }
-   if(file[0]=='/' || file[0]=='~' || u.path==0 || u.path[0]==0)
-   {
-      u.path=(char*)file;
-      u.Combine(buf);
-      return buf;
-   }
-   url::encode_string(file,buf,URL_PATH_UNSAFE);
-   return dir_file(url,buf);
+   u.path=(char*)dir_file(u.path,file);
+   xfree(buf);
+   buf=u.Combine();
+   buf_size=xstrlen(buf)+1;
+   return buf;
 }
 
 const char *output_file_name(const char *src,const char *dst,bool dst_local,
@@ -115,7 +117,8 @@ const char *output_file_name(const char *src,const char *dst,bool dst_local,
    {
       if(dst_base)
 	 dst=url_file(dst_base,dst);
-      if(url::is_url(dst))
+      ParsedURL u_dst(dst,true);
+      if(u_dst.proto)
 	 dst_local=false;
       if(dst_local)
       {
@@ -124,9 +127,12 @@ const char *output_file_name(const char *src,const char *dst,bool dst_local,
 	 if(stat(dst,&st)!=-1 && S_ISDIR(st.st_mode))
 	    dst_is_dir=true;
       }
-      int len=strlen(dst);
-      if(len>0 && dst[len-1]=='/')
-	 dst_is_dir=true;
+      else
+      {
+	 int len=xstrlen(u_dst.path);
+	 if(len>0 && u_dst.path[len-1]=='/')
+	    dst_is_dir=true;
+      }
       if(!dst_is_dir)
 	 return dst;
    }
@@ -422,9 +428,9 @@ int parse_year_or_time(const char *year_or_time,int *year,int *hour,int *minute)
    }
    return 0;
 }
-int guess_year(int month,int day,int hour,int minute,const struct tm *now_p)
+int guess_year(int month,int day,int hour,int minute)
 {
-   const struct tm &now=*now_p;
+   const struct tm &now=SMTask::now;
    int year=now.tm_year+1900;
    if(((month     *32+        day)*64+       hour)*64+       minute
     > ((now.tm_mon*32+now.tm_mday)*64+now.tm_hour)*64+now.tm_min)
@@ -587,6 +593,7 @@ char *Subst(const char *txt, const subst_t *s)
    char *store=buf;
 
    char str[3];
+   bool last_subst_empty=true;
 
    *store=0;
    while(*txt)
@@ -607,10 +614,19 @@ char *Subst(const char *txt, const subst_t *s)
 	    str[1]=0;
 	    to_add=str;
 	 } else {
+	    if(ch=='?')
+	    {
+	       if(last_subst_empty)
+		  txt++;
+	       to_add="";
+	    }
 	    for(int i = 0; s[i].from; i++) {
-	       if(s[i].from != ch) continue;
+	       if(s[i].from != ch)
+		  continue;
 	       to_add=s[i].to;
-	       if(!to_add) to_add = "";
+	       if(!to_add)
+		  to_add = "";
+	       last_subst_empty = (*to_add==0);
 	    }
 	    if(!to_add) {
 	       str[0]='\\';
@@ -721,7 +737,10 @@ char *xvasprintf(const char *format, va_list ap)
    for(;;)
    {
       ret = (char *) xrealloc(ret, siz);
-      int res=vsnprintf(ret, siz, format, ap);
+      va_list tmp;
+      VA_COPY(tmp,ap);
+      int res=vsnprintf(ret, siz, format, tmp);
+      va_end(tmp);
       if(res>=0 && res<siz)
 	 return ret;
       if(res>siz)
@@ -740,3 +759,30 @@ char *xasprintf(const char *format, ...)
    va_end(va);
    return ret;
 }
+
+/* /file/name -> /file
+ * /file -> /
+ * file/name -> "file"
+ * file/name/ -> "file"
+ * file -> ""
+ * note: last differs from dirname (wwould return ".")
+ *
+ */
+char *dirname_alloc(const char *fn)
+{
+   char *ret=xstrdup(fn);
+   int len=strlen(ret);
+
+   /* remove trailing slashes */
+   while(len>0 && ret[len-1]=='/')
+      ret[--len] = 0;
+
+   char *slash=strrchr(ret,'/');
+   if(!slash)
+      ret[0]=0; /* file with no path */
+   else if(slash==ret)
+      ret[1]=0; /* the slash is the first character */
+   else
+      *slash=0;
+   return ret;
+}
